@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ev-document-misc.h,v 1.1 2005/01/05 07:35:14 jrb Exp $
+ *  $Id: ev-document-misc.h,v 1.2 2005/02/02 13:22:52 jrb Exp $
  */
 
 #ifndef EV_DOCUMENT_MISC_H
@@ -32,6 +32,13 @@ GdkPixbuf *ev_document_misc_get_thumbnail_frame (int        width,
 						 int        height,
 						 GdkPixbuf *source_pixbuf);
 
+void ev_document_misc_get_page_border_size (gint  page_width,
+					    gint  page_height,
+					    gint *left_border,
+					    gint *right_border,
+					    gint *top_border,
+					    gint *bottom_border);
+
 G_END_DECLS
 
 #endif /* EV_DOCUMENT_MISC_H */
