@@ -27,8 +27,6 @@
 #include <glib-object.h>
 
 #include "ev-window.h"
-#include "ev-document.h"
-#include "ev-bookmark.h"
 
 G_BEGIN_DECLS
 
@@ -59,10 +57,6 @@ EvApplication	*ev_application_get_instance		(void);
 void		 ev_application_open			(EvApplication *application,
 							 GError        *err);
 EvWindow	*ev_application_new_window		(EvApplication *application);
-void		 ev_application_open_bookmark		(EvApplication *application,
-							 EvDocument    *document,
-							 EvBookmark    *bookmark,
-							 GError        *err);
 
 G_END_DECLS
 
