@@ -21,6 +21,7 @@
 #include "ev-page-view.h"
 #include "ev-marshal.h"
 #include "ev-document-misc.h"
+#include "ev-debug.h"
 #include <gtk/gtk.h>
 
 /* We keep a cached array of all the page sizes.  The info is accessed via
@@ -97,6 +98,21 @@ ev_page_view_init (EvPageView *page_view)
 	page_view->priv->uniform_page_size = FALSE;
 }
 
+static void
+ev_page_view_dispose (GObject *object)
+{
+	EvPageView *view = EV_PAGE_VIEW (object);
+
+	LOG ("dispose")
+
+	if (view->priv->document) {
+		g_object_unref (view->priv->document);
+		view->priv->document = NULL;
+	}
+
+	G_OBJECT_CLASS (ev_page_view_parent_class)->dispose (object);
+}
+
 static void
 ev_page_view_class_init (EvPageViewClass *klass)
 {
@@ -115,6 +131,8 @@ ev_page_view_class_init (EvPageViewClass *klass)
 	widget_class->unrealize = ev_page_view_unrealize;
 	widget_class->map = ev_page_view_map;
 
+	o_class->dispose = ev_page_view_dispose;
+
 	widget_class->set_scroll_adjustments_signal =
 		g_signal_new ("set_scroll_adjustments",
 			      G_TYPE_FROM_CLASS (o_class),
