@@ -4,7 +4,7 @@
  *  Copyright (C) 2004 Martin Kretzschmar
  *  Copyright (C) 2004 Red Hat, Inc.
  *  Copyright (C) 2000, 2001, 2002, 2003, 2004 Marco Pesenti Gritti
- *  Copyright (C) 2003, 2004 Christian Persch
+ *  Copyright (C) 2003, 2004, 2005 Christian Persch
  *
  *  Author:
  *    Martin Kretzschmar <martink@gnome.org>
@@ -34,6 +34,7 @@
 #include "ev-sidebar-links.h"
 #include "ev-sidebar-thumbnails.h"
 #include "ev-view.h"
+#include "ev-page-view.h"
 #include "ev-password.h"
 #include "ev-password-view.h"
 #include "ev-print-job.h"
@@ -51,6 +52,8 @@
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 #include <libgnomeprintui/gnome-print-dialog.h>
 
+#include <gconf/gconf-client.h>
+
 #include <string.h>
 
 #include "ev-application.h"
@@ -62,14 +65,27 @@ typedef enum {
 	PAGE_MODE_PASSWORD,
 } EvWindowPageMode;
 
+typedef enum {
+	EV_CHROME_MENUBAR	= 1 << 0,
+	EV_CHROME_TOOLBAR	= 1 << 1,
+	EV_CHROME_SIDEBAR	= 1 << 2,
+	EV_CHROME_FINDBAR	= 1 << 3,
+	EV_CHROME_STATUSBAR	= 1 << 4,
+	EV_CHROME_NORMAL	= EV_CHROME_MENUBAR | EV_CHROME_TOOLBAR | EV_CHROME_SIDEBAR | EV_CHROME_STATUSBAR
+} EvChrome;
+
 struct _EvWindowPrivate {
 	GtkWidget *main_box;
+	GtkWidget *menubar;
+	GtkWidget *toolbar_dock;
+	GtkWidget *toolbar;
 	GtkWidget *hpaned;
 	GtkWidget *sidebar;
 	GtkWidget *thumbs_sidebar;
 	GtkWidget *find_bar;
 	GtkWidget *scrolled_window;
 	GtkWidget *view;
+	GtkWidget *page_view;
 	GtkWidget *password_view;
 	GtkActionGroup *action_group;
 	GtkUIManager *ui_manager;
@@ -77,23 +93,28 @@ struct _EvWindowPrivate {
 	guint help_message_cid;
 	guint view_message_cid;
 	GtkWidget *exit_fullscreen_popup;
+	GtkWidget *exit_fullscreen_toolbar;
 	char *uri;
 
 	EvDocument *document;
 
+	EvWindowPageMode page_mode;
 	/* These members are used temporarily when in PAGE_MODE_PASSWORD */
 	EvDocument *password_document;
 	GtkWidget *password_dialog;
 	char *password_uri;
 
+	EvChrome chrome;
 	gboolean fullscreen_mode;
 };
 #define EV_WINDOW_GET_PRIVATE(object) \
 	(G_TYPE_INSTANCE_GET_PRIVATE ((object), EV_TYPE_WINDOW, EvWindowPrivate))
 
+#define PAGE_SELECTOR_ACTION	"PageSelector"
 
-#define PAGE_SELECTOR_ACTION "PageSelector"
-
+#define GCONF_CHROME_TOOLBAR	"/apps/evince/show_toolbar"
+#define GCONF_CHROME_SIDEBAR	"/apps/evince/show_sidebar"
+#define GCONF_CHROME_STATUSBAR	"/apps/evince/show_statusbar"
 
 static void     ev_window_update_fullscreen_popup (EvWindow         *window);
 static void     ev_window_set_page_mode           (EvWindow         *window,
@@ -119,9 +140,11 @@ static void
 update_action_sensitivity (EvWindow *ev_window)
 {
 	EvDocument *document;
+	EvWindowPageMode page_mode;
 	EvView *view;
 
 	document = ev_window->priv->document;
+	page_mode = ev_window->priv->page_mode;
 
 	view = EV_VIEW (ev_window->priv->view);
 
@@ -155,17 +178,25 @@ update_action_sensitivity (EvWindow *ev_window)
 		page = ev_view_get_page (EV_VIEW (ev_window->priv->view));
 		n_pages = ev_document_get_n_pages (document);
 
-		set_action_sensitive (ev_window, "GoPageUp", page > 1);
-		set_action_sensitive (ev_window, "GoPageDown", page < n_pages);
+		set_action_sensitive (ev_window, "GoPreviousPage", page > 1);
+		set_action_sensitive (ev_window, "GoNextPage", page < n_pages);
 		set_action_sensitive (ev_window, "GoFirstPage", page > 1);
 		set_action_sensitive (ev_window, "GoLastPage", page < n_pages);
 	} else {
-		set_action_sensitive (ev_window, "GoFirstPage", FALSE);
-		set_action_sensitive (ev_window, "GoPageUp", FALSE);
-		set_action_sensitive (ev_window, "GoPageDown", FALSE);
+  		set_action_sensitive (ev_window, "GoFirstPage", FALSE);
+		set_action_sensitive (ev_window, "GoPreviousPage", FALSE);
+		set_action_sensitive (ev_window, "GoNextPage", FALSE);
 		set_action_sensitive (ev_window, "GoLastPage", FALSE);
 	}
 
+	/* Page View radio group */
+	if (document) {
+		set_action_sensitive (ev_window, "SinglePage", page_mode != PAGE_MODE_PASSWORD);
+		set_action_sensitive (ev_window, "ContinuousPage", page_mode != PAGE_MODE_PASSWORD);
+	} else {
+		set_action_sensitive (ev_window, "SinglePage", FALSE);
+		set_action_sensitive (ev_window, "ContinuousPage", FALSE);
+	}
 	/* Help menu */
 	/* "HelpContents": always sensitive */
 	/* "HelpAbout": always sensitive */
@@ -174,6 +205,52 @@ update_action_sensitivity (EvWindow *ev_window)
 	set_action_sensitive (ev_window, PAGE_SELECTOR_ACTION, document!=NULL);
 }
 
+static void
+update_chrome_visibility (EvWindow *window)
+{
+	EvWindowPrivate *priv = window->priv;
+	gboolean menubar, toolbar, sidebar, findbar, statusbar;
+
+	menubar = (priv->chrome & EV_CHROME_MENUBAR) != 0 && !priv->fullscreen_mode;
+	toolbar = (priv->chrome & EV_CHROME_TOOLBAR) != 0;
+	sidebar = (priv->chrome & EV_CHROME_SIDEBAR) != 0;
+	findbar = (priv->chrome & EV_CHROME_FINDBAR) != 0;
+	statusbar = (priv->chrome & EV_CHROME_STATUSBAR) != 0 && !priv->fullscreen_mode;
+
+	g_object_set (priv->menubar, "visible", menubar, NULL);
+	g_object_set (priv->toolbar_dock, "visible", toolbar, NULL);
+	g_object_set (priv->sidebar, "visible", sidebar, NULL);
+	g_object_set (priv->find_bar, "visible", findbar, NULL);
+	g_object_set (priv->statusbar, "visible", statusbar, NULL);
+
+	g_object_set (priv->exit_fullscreen_toolbar, "visible", priv->fullscreen_mode, NULL);
+	if (priv->exit_fullscreen_popup != NULL) {
+		g_object_set (priv->exit_fullscreen_popup, "visible", !toolbar, NULL);
+	}
+}
+
+static void
+update_chrome_flag (EvWindow *window, EvChrome flag, const char *pref, gboolean active)
+{
+	EvWindowPrivate *priv = window->priv;
+	GConfClient *client;
+	
+	if (active) {
+		priv->chrome |= flag;
+	}
+	else {
+		priv->chrome &= ~flag;
+	}
+
+	if (pref != NULL) {
+		client = gconf_client_get_default ();
+		gconf_client_set_bool (client, pref, active, NULL);
+		g_object_unref (client);
+	}
+
+	update_chrome_visibility (window);
+}
+
 void
 ev_window_open_page (EvWindow *ev_window, int page)
 {
@@ -313,6 +390,7 @@ ev_window_setup_document (EvWindow *ev_window)
 {
 	EvDocument *document;
 	EvView *view = EV_VIEW (ev_window->priv->view);
+	//EvPageView *page_view = EV_PAGE_VIEW (ev_window->priv->page_view);
 	EvSidebar *sidebar = EV_SIDEBAR (ev_window->priv->sidebar);
 
 	document = ev_window->priv->document;
@@ -326,8 +404,9 @@ ev_window_setup_document (EvWindow *ev_window)
 
 	ev_sidebar_set_document (sidebar, document);
 	ev_view_set_document (view, document);
+	//ev_page_view_set_document (page_view, document);
 
-	update_window_title (ev_window->priv->document, NULL, ev_window);
+	update_window_title (document, NULL, ev_window);
 	update_total_pages (ev_window);
 	update_action_sensitivity (ev_window);
 }
@@ -482,7 +561,9 @@ ev_window_open (EvWindow *ev_window, const char *uri)
 		document = NULL;
 	else if (!strcmp (mime_type, "application/pdf"))
 		document = g_object_new (PDF_TYPE_DOCUMENT, NULL);
-	else if (!strcmp (mime_type, "application/postscript"))
+	else if (!strcmp (mime_type, "application/postscript") ||
+		 !strcmp (mime_type, "application/x-gzpostscript") ||
+		 !strcmp (mime_type, "image/x-eps"))
 		document = g_object_new (PS_TYPE_DOCUMENT, NULL);
 	else if (mime_type_supported_by_gdk_pixbuf (mime_type))
 		document = g_object_new (PIXBUF_TYPE_DOCUMENT, NULL);
@@ -766,10 +847,7 @@ ev_window_cmd_edit_find (GtkAction *action, EvWindow *ev_window)
 	} else if (!EV_IS_DOCUMENT_FIND (ev_window->priv->document)) {
 		find_not_supported_dialog (ev_window);
 	} else {
-		gtk_widget_show (ev_window->priv->find_bar);
-
-		if (ev_window->priv->exit_fullscreen_popup)
-			ev_window_update_fullscreen_popup (ev_window);
+		update_chrome_flag (ev_window, EV_CHROME_FINDBAR, NULL, TRUE);
 
 		egg_find_bar_grab_focus (EGG_FIND_BAR (ev_window->priv->find_bar));
 	}
@@ -792,9 +870,6 @@ ev_window_update_fullscreen_popup (EvWindow *window)
 
 	g_return_if_fail (popup != NULL);
 
-	if (!popup)
-		return;
-
 	popup_width = popup->requisition.width;
 	popup_height = popup->requisition.height;
 
@@ -805,24 +880,17 @@ ev_window_update_fullscreen_popup (EvWindow *window)
                          GTK_WIDGET (window)->window),
                          &screen_rect);
 
-	if (GTK_WIDGET_VISIBLE (window->priv->find_bar)) {
-		GtkRequisition req;
-
-		gtk_widget_size_request (window->priv->find_bar, &req);
-
-		screen_rect.height -= req.height;
-	}
-
 	if (gtk_widget_get_direction (popup) == GTK_TEXT_DIR_RTL)
 	{
 		gtk_window_move (GTK_WINDOW (popup),
-				 screen_rect.x + screen_rect.width - popup_width,
-				 screen_rect.height - popup_height);
+				 screen_rect.x,
+				 screen_rect.y);
 	}
 	else
 	{
 		gtk_window_move (GTK_WINDOW (popup),
-                	        screen_rect.x, screen_rect.height - popup_height);
+				 screen_rect.x + screen_rect.width - popup_width,
+				 screen_rect.y);
 	}
 }
 
@@ -868,10 +936,12 @@ fullscreen_popup_size_request_cb (GtkWidget *popup, GtkRequisition *req, EvWindo
 static void
 ev_window_fullscreen (EvWindow *window)
 {
-	GtkWidget *popup, *button, *icon, *label, *hbox, *main_menu;
+	GtkWidget *popup, *button, *icon, *label, *hbox;
 
 	window->priv->fullscreen_mode = TRUE;
 
+	g_return_if_fail (window->priv->exit_fullscreen_popup == NULL);
+
 	popup = gtk_window_new (GTK_WINDOW_POPUP);
 	window->priv->exit_fullscreen_popup = popup;
 
@@ -886,11 +956,11 @@ ev_window_fullscreen (EvWindow *window)
 	gtk_widget_show (hbox);
 	gtk_container_add (GTK_CONTAINER (button), hbox);
 
-	icon = gtk_image_new_from_stock (GTK_STOCK_QUIT, GTK_ICON_SIZE_BUTTON);
+	icon = gtk_image_new_from_stock (EV_STOCK_LEAVE_FULLSCREEN, GTK_ICON_SIZE_BUTTON);
 	gtk_widget_show (icon);
 	gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, FALSE, 0);
 
-	label = gtk_label_new (_("Exit Fullscreen"));
+	label = gtk_label_new (_("Leave Fullscreen"));
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
@@ -902,27 +972,19 @@ ev_window_fullscreen (EvWindow *window)
 	g_signal_connect (popup, "size_request",
 			  G_CALLBACK (fullscreen_popup_size_request_cb), window);
 
-	main_menu = gtk_ui_manager_get_widget (window->priv->ui_manager, "/MainMenu");
-	gtk_widget_hide (main_menu);
-	gtk_widget_hide (window->priv->statusbar);
+	update_chrome_visibility (window);
 
 	ev_window_update_fullscreen_popup (window);
-
-	gtk_widget_show (popup);
 }
 
 static void
 ev_window_unfullscreen (EvWindow *window)
 {
-	GtkWidget *main_menu;
-
 	window->priv->fullscreen_mode = FALSE;
 
-	main_menu = gtk_ui_manager_get_widget (window->priv->ui_manager, "/MainMenu");
-	gtk_widget_show (main_menu);
-	gtk_widget_show (window->priv->statusbar);
-
 	destroy_exit_fullscreen_popup (window);
+
+	update_chrome_visibility (window);
 }
 
 static void
@@ -976,11 +1038,33 @@ ev_window_state_event_cb (GtkWidget *widget, GdkEventWindowState *event, EvWindo
 }
 
 static gboolean
-ev_window_focus_out_cb (GtkWidget *widget, GdkEventFocus *event, EvWindow *ev_window)
+ev_window_focus_in_event (GtkWidget *widget, GdkEventFocus *event)
 {
-	gtk_window_unfullscreen (GTK_WINDOW (ev_window));
+	EvWindow *window = EV_WINDOW (widget);
+	EvWindowPrivate *priv = window->priv;
 
-	return FALSE;
+	if (priv->exit_fullscreen_popup != NULL &&
+	    (priv->chrome & EV_CHROME_TOOLBAR) == 0)
+	{
+		gtk_widget_show (priv->exit_fullscreen_popup);
+	}
+
+	return GTK_WIDGET_CLASS (ev_window_parent_class)->focus_in_event (widget, event);
+}
+
+static gboolean
+ev_window_focus_out_event (GtkWidget *widget, GdkEventFocus *event)
+{
+	EvWindow *window = EV_WINDOW (widget);
+	EvWindowPrivate *priv = window->priv;
+
+	if (priv->exit_fullscreen_popup != NULL &&
+	    (priv->chrome & EV_CHROME_TOOLBAR) == 0)
+	{
+		gtk_widget_hide (priv->exit_fullscreen_popup);
+	}
+
+	return GTK_WIDGET_CLASS (ev_window_parent_class)->focus_out_event (widget, event);
 }
 
 static void
@@ -990,6 +1074,11 @@ ev_window_set_page_mode (EvWindow         *window,
 	GtkWidget *child = NULL;
 	GtkWidget *real_child;
 
+	if (window->priv->page_mode == page_mode)
+		return;
+
+	window->priv->page_mode = page_mode;
+
 	switch (page_mode) {
 	case PAGE_MODE_SINGLE_PAGE:
 		child = window->priv->view;
@@ -997,10 +1086,13 @@ ev_window_set_page_mode (EvWindow         *window,
 	case PAGE_MODE_PASSWORD:
 		child = window->priv->password_view;
 		break;
+	case PAGE_MODE_CONTINUOUS_PAGE:
+		child = window->priv->page_view;
+		break;
 	default:
-		g_warning ("page_mode not implemented yet\n");
 		g_assert_not_reached ();
 	}
+
 	real_child = gtk_bin_get_child (GTK_BIN (window->priv->scrolled_window));
 	if (child != real_child) {
 		gtk_container_remove (GTK_CONTAINER (window->priv->scrolled_window),
@@ -1008,6 +1100,7 @@ ev_window_set_page_mode (EvWindow         *window,
 		gtk_container_add (GTK_CONTAINER (window->priv->scrolled_window),
 				   child);
 	}
+	update_action_sensitivity (window);
 }
 
 static void
@@ -1051,7 +1144,7 @@ ev_window_cmd_view_page_width (GtkAction *action, EvWindow *ev_window)
 }
 
 static void
-ev_window_cmd_go_page_up (GtkAction *action, EvWindow *ev_window)
+ev_window_cmd_go_previous_page (GtkAction *action, EvWindow *ev_window)
 {
         g_return_if_fail (EV_IS_WINDOW (ev_window));
 
@@ -1060,7 +1153,7 @@ ev_window_cmd_go_page_up (GtkAction *action, EvWindow *ev_window)
 }
 
 static void
-ev_window_cmd_go_page_down (GtkAction *action, EvWindow *ev_window)
+ev_window_cmd_go_next_page (GtkAction *action, EvWindow *ev_window)
 {
         g_return_if_fail (EV_IS_WINDOW (ev_window));
 
@@ -1092,6 +1185,12 @@ ev_window_cmd_help_contents (GtkAction *action, EvWindow *ev_window)
         /* FIXME */
 }
 
+static void
+ev_window_cmd_leave_fullscreen (GtkAction *action, EvWindow *window)
+{
+	gtk_window_unfullscreen (GTK_WINDOW (window));
+}
+
 static void
 ev_window_cmd_help_about (GtkAction *action, EvWindow *ev_window)
 {
@@ -1154,33 +1253,25 @@ ev_window_cmd_help_about (GtkAction *action, EvWindow *ev_window)
 static void
 ev_window_view_toolbar_cb (GtkAction *action, EvWindow *ev_window)
 {
-	g_object_set (
-		G_OBJECT (gtk_ui_manager_get_widget (
-				  ev_window->priv->ui_manager,
-				  "/ToolBar")),
-		"visible",
-		gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)),
-		NULL);
+	update_chrome_flag (ev_window, EV_CHROME_TOOLBAR,
+			    GCONF_CHROME_TOOLBAR,
+			    gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
 }
 
 static void
 ev_window_view_statusbar_cb (GtkAction *action, EvWindow *ev_window)
 {
-	g_object_set (
-		ev_window->priv->statusbar,
-		"visible",
-		gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)),
-		NULL);
+	update_chrome_flag (ev_window, EV_CHROME_STATUSBAR,
+			    GCONF_CHROME_STATUSBAR,
+			    gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
 }
 
 static void
 ev_window_view_sidebar_cb (GtkAction *action, EvWindow *ev_window)
 {
-	g_object_set (
-		ev_window->priv->sidebar,
-		"visible",
-		gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)),
-		NULL);
+	update_chrome_flag (ev_window, EV_CHROME_SIDEBAR,
+			    GCONF_CHROME_SIDEBAR,
+			    gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
 }
 
 static void
@@ -1305,10 +1396,22 @@ static void
 find_bar_close_cb (EggFindBar *find_bar,
 		   EvWindow   *ev_window)
 {
-	gtk_widget_hide (ev_window->priv->find_bar);
+	update_chrome_flag (ev_window, EV_CHROME_FINDBAR, NULL, FALSE);
+}
 
-	if (ev_window->priv->exit_fullscreen_popup)
-		ev_window_update_fullscreen_popup (ev_window);
+static void
+ev_window_page_mode_cb (GtkRadioAction *action,
+			GtkRadioAction *activated_action,
+			EvWindow       *window)
+{
+	int mode;
+
+	mode = gtk_radio_action_get_current_value (action);
+
+	g_assert (mode == PAGE_MODE_CONTINUOUS_PAGE ||
+		  mode == PAGE_MODE_SINGLE_PAGE);
+
+	ev_window_set_page_mode (window, (EvWindowPageMode) mode);
 }
 
 static void
@@ -1344,6 +1447,7 @@ find_bar_search_changed_cb (EggFindBar *find_bar,
 			ev_document_find_cancel (EV_DOCUMENT_FIND (ev_window->priv->document));
 			egg_find_bar_set_status_text (EGG_FIND_BAR (ev_window->priv->find_bar),
 						      NULL);
+			gtk_widget_queue_draw (GTK_WIDGET (ev_window->priv->view));
 		}
 	}
 }
@@ -1351,11 +1455,8 @@ find_bar_search_changed_cb (EggFindBar *find_bar,
 static void
 ev_window_dispose (GObject *object)
 {
-	EvWindowPrivate *priv;
-
-	g_return_if_fail (object != NULL && EV_IS_WINDOW (object));
-
-	priv = EV_WINDOW (object)->priv;
+	EvWindow *window = EV_WINDOW (object);
+	EvWindowPrivate *priv = window->priv;
 
 	if (priv->ui_manager) {
 		g_object_unref (priv->ui_manager);
@@ -1371,7 +1472,17 @@ ev_window_dispose (GObject *object)
 		g_object_unref (priv->document);
 		priv->document = NULL;
 	}
-	
+
+	if (priv->view) {
+		g_object_unref (priv->view);
+		priv->view = NULL;
+	}
+
+	if (priv->page_view) {
+		g_object_unref (priv->page_view);
+		priv->page_view = NULL;
+	}
+
 	if (priv->password_document) {
 		g_object_unref (priv->password_document);
 		priv->password_document = NULL;
@@ -1381,20 +1492,24 @@ ev_window_dispose (GObject *object)
 		g_free (priv->password_uri);
 		priv->password_uri = NULL;
 	}
-	
+
+	destroy_exit_fullscreen_popup (window);
+
 	G_OBJECT_CLASS (ev_window_parent_class)->dispose (object);
 }
 
 static void
 ev_window_class_init (EvWindowClass *ev_window_class)
 {
-	GObjectClass *g_object_class;
+	GObjectClass *g_object_class = G_OBJECT_CLASS (ev_window_class);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (ev_window_class);
 
-	g_object_class = G_OBJECT_CLASS (ev_window_class);
 	g_object_class->dispose = ev_window_dispose;
 
-	g_type_class_add_private (g_object_class, sizeof (EvWindowPrivate));
+	widget_class->focus_in_event = ev_window_focus_in_event;
+	widget_class->focus_out_event = ev_window_focus_out_event;
 
+	g_type_class_add_private (g_object_class, sizeof (EvWindowPrivate));
 }
 
 /* Normal items */
@@ -1406,54 +1521,54 @@ static GtkActionEntry entries[] = {
 	{ "Help", NULL, N_("_Help") },
 
 	/* File menu */
-	{ "FileOpen", GTK_STOCK_OPEN, N_("_Open"), "<control>O",
-	  N_("Open a file"),
+	{ "FileOpen", GTK_STOCK_OPEN, NULL, "<control>O",
+	  N_("Open an existing document"),
 	  G_CALLBACK (ev_window_cmd_file_open) },
        	{ "FileSaveAs", GTK_STOCK_SAVE_AS, N_("_Save a Copy..."), NULL,
 	  N_("Save the current document with a new filename"),
 	  G_CALLBACK (ev_window_cmd_save_as) },
-	{ "FilePrint", GTK_STOCK_PRINT, N_("_Print"), "<control>P",
+	{ "FilePrint", GTK_STOCK_PRINT, NULL, "<control>P",
 	  N_("Print this document"),
 	  G_CALLBACK (ev_window_cmd_file_print) },
-	{ "FileCloseWindow", GTK_STOCK_CLOSE, N_("_Close"), "<control>W",
+	{ "FileCloseWindow", GTK_STOCK_CLOSE, NULL, "<control>W",
 	  N_("Close this window"),
 	  G_CALLBACK (ev_window_cmd_file_close_window) },
 
         /* Edit menu */
-        { "EditCopy", GTK_STOCK_COPY, N_("_Copy"), "<control>C",
+        { "EditCopy", GTK_STOCK_COPY, NULL, "<control>C",
           N_("Copy text from the document"),
           G_CALLBACK (ev_window_cmd_edit_copy) },
  	{ "EditSelectAll", NULL, N_("Select _All"), "<control>A",
 	  N_("Select the entire page"),
 	  G_CALLBACK (ev_window_cmd_edit_select_all) },
-        { "EditFind", GTK_STOCK_FIND, N_("_Find"), "<control>F",
+        { "EditFind", GTK_STOCK_FIND, NULL, "<control>F",
           N_("Find a word or phrase in the document"),
           G_CALLBACK (ev_window_cmd_edit_find) },
 
         /* View menu */
-        { "ViewZoomIn", GTK_STOCK_ZOOM_IN, N_("Zoom _In"), "<control>plus",
+        { "ViewZoomIn", GTK_STOCK_ZOOM_IN, NULL, "<control>plus",
           N_("Enlarge the document"),
           G_CALLBACK (ev_window_cmd_view_zoom_in) },
-        { "ViewZoomOut", GTK_STOCK_ZOOM_OUT, N_("Zoom _Out"), "<control>minus",
+        { "ViewZoomOut", GTK_STOCK_ZOOM_OUT, NULL, "<control>minus",
           N_("Shrink the document"),
           G_CALLBACK (ev_window_cmd_view_zoom_out) },
-        { "ViewNormalSize", GTK_STOCK_ZOOM_100, N_("_Normal Size"), "<control>0",
-          N_("Zoom to the normal size"),
+        { "ViewNormalSize", GTK_STOCK_ZOOM_100, NULL, "<control>0",
+          N_("Reset the zoom level to the defaul value"),
           G_CALLBACK (ev_window_cmd_view_normal_size) },
-        { "ViewBestFit", GTK_STOCK_ZOOM_FIT, N_("_Best Fit"), NULL,
-          N_("Zoom to fit the document to the current window"),
+        { "ViewBestFit", EV_STOCK_ZOOM_PAGE, N_("_Best Fit"), NULL,
+          N_("Make the current document fill the window"),
           G_CALLBACK (ev_window_cmd_view_best_fit) },
-        { "ViewPageWidth", EV_STOCK_ZOOM_FIT_WIDTH, N_("Fit Page _Width"), NULL,
-          N_("Zoom to fit the width of the current window "),
+        { "ViewPageWidth", EV_STOCK_ZOOM_WIDTH, N_("Fit Page _Width"), NULL,
+          N_("Make the current document fill the window width"),
           G_CALLBACK (ev_window_cmd_view_page_width) },
 
         /* Go menu */
-        { "GoPageUp", GTK_STOCK_GO_UP, N_("_Page Up"), "<control>Page_Up",
+        { "GoPreviousPage", GTK_STOCK_GO_BACK, N_("_Previous Page"), "Page_Up",
           N_("Go to the previous page"),
-          G_CALLBACK (ev_window_cmd_go_page_up) },
-        { "GoPageDown", GTK_STOCK_GO_DOWN, N_("_Page Down"), "<control>Page_Down",
+          G_CALLBACK (ev_window_cmd_go_previous_page) },
+        { "GoNextPage", GTK_STOCK_GO_FORWARD, N_("_Next Page"), "Page_Down",
           N_("Go to the next page"),
-          G_CALLBACK (ev_window_cmd_go_page_down) },
+          G_CALLBACK (ev_window_cmd_go_next_page) },
         { "GoFirstPage", GTK_STOCK_GOTO_FIRST, N_("_First Page"), "<control>Home",
           N_("Go to the first page"),
           G_CALLBACK (ev_window_cmd_go_first_page) },
@@ -1469,25 +1584,39 @@ static GtkActionEntry entries[] = {
 	{ "HelpAbout", GTK_STOCK_ABOUT, N_("_About"), NULL,
 	  N_("Display credits for the document viewer creators"),
 	  G_CALLBACK (ev_window_cmd_help_about) },
+
+	/* Toolbar-only */
+	{ "LeaveFullscreen", EV_STOCK_LEAVE_FULLSCREEN, N_("Leave Fullscreen"), "Escape",
+	  N_("Leave fullscreen mode"),
+	  G_CALLBACK (ev_window_cmd_leave_fullscreen) }
 };
 
 /* Toggle items */
 static GtkToggleActionEntry toggle_entries[] = {
 	/* View Menu */
 	{ "ViewToolbar", NULL, N_("_Toolbar"), "<shift><control>T",
-	  N_("Show or hide toolbar"),
+	  N_("Show or hide the toolbar"),
 	  G_CALLBACK (ev_window_view_toolbar_cb), TRUE },
 	{ "ViewStatusbar", NULL, N_("_Statusbar"), NULL,
-	  N_("Show or hide statusbar"),
+	  N_("Show or hide the statusbar"),
 	  G_CALLBACK (ev_window_view_statusbar_cb), TRUE },
-        { "ViewSidebar", NULL, N_("Side_bar"), "F9",
-	  N_("Show or hide sidebar"),
+        { "ViewSidebar", NULL, N_("Side _pane"), "F9",
+	  N_("Show or hide the side pane"),
 	  G_CALLBACK (ev_window_view_sidebar_cb), TRUE },
         { "ViewFullscreen", NULL, N_("_Fullscreen"), "F11",
           N_("Expand the window to fill the screen"),
           G_CALLBACK (ev_window_cmd_view_fullscreen) },
 };
 
+static GtkRadioActionEntry page_view_entries[] = {
+	{ "SinglePage", GTK_STOCK_DND, N_("Single"), NULL,
+	  N_("Show the document one page at a time"),
+	  PAGE_MODE_SINGLE_PAGE },
+	{ "ContinuousPage", GTK_STOCK_DND_MULTIPLE, N_("Multi"), NULL,
+	  N_("Show the full document at once"),
+	  PAGE_MODE_CONTINUOUS_PAGE }
+};
+
 static void
 goto_page_cb (GtkAction *action, int page_number, EvWindow *ev_window)
 {
@@ -1515,16 +1644,93 @@ register_custom_actions (EvWindow *window, GtkActionGroup *group)
 }
 
 static void
-set_short_labels (GtkActionGroup *action_group)
+set_action_properties (GtkActionGroup *action_group)
 {
 	GtkAction *action;
 
-	action = gtk_action_group_get_action (action_group, "GoPageUp");
-	g_object_set (action, "short_label", _("Up"), NULL);
-	action = gtk_action_group_get_action (action_group, "GoPageDown");
-	g_object_set (action, "short_label", _("Down"), NULL);
+	action = gtk_action_group_get_action (action_group, "GoPreviousPage");
+	/*translators: this is the label for toolbar button*/
+	g_object_set (action, "short_label", _("Previous"), NULL);
+	action = gtk_action_group_get_action (action_group, "GoNextPage");
+	/*translators: this is the label for toolbar button*/
+	g_object_set (action, "short_label", _("Next"), NULL);
 	action = gtk_action_group_get_action (action_group, "ViewPageWidth");
+	/*translators: this is the label for toolbar button*/
 	g_object_set (action, "short_label", _("Fit Width"), NULL);
+	action = gtk_action_group_get_action (action_group, "ViewZoomIn");
+
+	action = gtk_action_group_get_action (action_group, "LeaveFullscreen");
+	g_object_set (action, "is-important", TRUE, NULL);
+}
+
+static void
+set_chrome_actions (EvWindow *window)
+{
+	EvWindowPrivate *priv = window->priv;
+	GtkActionGroup *action_group = priv->action_group;
+	GtkAction *action;
+
+	action= gtk_action_group_get_action (action_group, "ViewToolbar");
+	g_signal_handlers_block_by_func
+		(action, G_CALLBACK (ev_window_view_toolbar_cb), window);
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
+				      (priv->chrome & EV_CHROME_TOOLBAR) != 0);
+	g_signal_handlers_unblock_by_func
+		(action, G_CALLBACK (ev_window_view_toolbar_cb), window);
+
+	action= gtk_action_group_get_action (action_group, "ViewSidebar");
+	g_signal_handlers_block_by_func
+		(action, G_CALLBACK (ev_window_view_sidebar_cb), window);
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
+				      (priv->chrome & EV_CHROME_SIDEBAR) != 0);
+	g_signal_handlers_unblock_by_func
+		(action, G_CALLBACK (ev_window_view_sidebar_cb), window);
+
+	action= gtk_action_group_get_action (action_group, "ViewStatusbar");
+	g_signal_handlers_block_by_func
+		(action, G_CALLBACK (ev_window_view_statusbar_cb), window);
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
+				      (priv->chrome & EV_CHROME_STATUSBAR) != 0);
+	g_signal_handlers_unblock_by_func
+		(action, G_CALLBACK (ev_window_view_statusbar_cb), window);
+}
+
+static EvChrome
+load_chrome (void)
+{
+	GConfClient *client;
+	GConfValue *value;	
+	EvChrome chrome = EV_CHROME_NORMAL;
+
+	client = gconf_client_get_default ();
+
+	value = gconf_client_get (client, GCONF_CHROME_TOOLBAR, NULL);
+	if (value != NULL) {
+		if (value->type == GCONF_VALUE_BOOL && !gconf_value_get_bool (value)) {
+			chrome &= ~EV_CHROME_TOOLBAR;
+		}
+		gconf_value_free (value);
+	}
+
+	value = gconf_client_get (client, GCONF_CHROME_SIDEBAR, NULL);
+	if (value != NULL) {
+		if (value->type == GCONF_VALUE_BOOL && !gconf_value_get_bool (value)) {
+			chrome &= ~EV_CHROME_SIDEBAR;
+		}
+		gconf_value_free (value);
+	}
+
+	value = gconf_client_get (client, GCONF_CHROME_STATUSBAR, NULL);
+	if (value != NULL) {
+		if (value->type == GCONF_VALUE_BOOL && !gconf_value_get_bool (value)) {
+			chrome &= ~EV_CHROME_STATUSBAR;
+		}
+		gconf_value_free (value);
+	}
+
+	g_object_unref (client);
+
+	return chrome;
 }
 
 static void
@@ -1533,12 +1739,11 @@ ev_window_init (EvWindow *ev_window)
 	GtkActionGroup *action_group;
 	GtkAccelGroup *accel_group;
 	GError *error = NULL;
-	GtkWidget *menubar;
-	GtkWidget *toolbar;
-	GtkWidget *sidebar_widget;
+	GtkWidget *sidebar_widget, *toolbar_dock;
 
 	ev_window->priv = EV_WINDOW_GET_PRIVATE (ev_window);
 
+	ev_window->priv->page_mode = PAGE_MODE_SINGLE_PAGE;
 	update_window_title (NULL, NULL, ev_window);
 
 	ev_window->priv->main_box = gtk_vbox_new (FALSE, 0);
@@ -1553,7 +1758,12 @@ ev_window_init (EvWindow *ev_window)
 	gtk_action_group_add_toggle_actions (action_group, toggle_entries,
 					     G_N_ELEMENTS (toggle_entries),
 					     ev_window);
-	set_short_labels (action_group);
+	gtk_action_group_add_radio_actions (action_group, page_view_entries,
+					    G_N_ELEMENTS (page_view_entries),
+					    ev_window->priv->page_mode,
+					    G_CALLBACK (ev_window_page_mode_cb),
+					    ev_window);
+	set_action_properties (action_group);
 	register_custom_actions (ev_window, action_group);
 
 	ev_window->priv->ui_manager = gtk_ui_manager_new ();
@@ -1576,15 +1786,39 @@ ev_window_init (EvWindow *ev_window)
 		g_error_free (error);
 	}
 
-	menubar = gtk_ui_manager_get_widget (ev_window->priv->ui_manager,
-					     "/MainMenu");
-	gtk_box_pack_start (GTK_BOX (ev_window->priv->main_box), menubar,
+	ev_window->priv->menubar =
+		 gtk_ui_manager_get_widget (ev_window->priv->ui_manager,
+					    "/MainMenu");
+	gtk_box_pack_start (GTK_BOX (ev_window->priv->main_box),
+			    ev_window->priv->menubar,
 			    FALSE, FALSE, 0);
 
-	toolbar = gtk_ui_manager_get_widget (ev_window->priv->ui_manager,
-					     "/ToolBar");
-	gtk_box_pack_start (GTK_BOX (ev_window->priv->main_box), toolbar,
+	/* This sucks, but there is no way to have a draw=no, expand=true separator
+	 * in a GtkUIManager-built toolbar. So, just add another toolbar.
+	 * See gtk+ bug 166489.
+	 */
+	toolbar_dock = ev_window->priv->toolbar_dock = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (ev_window->priv->main_box), toolbar_dock,
 			    FALSE, FALSE, 0);
+	gtk_widget_show (toolbar_dock);
+
+	ev_window->priv->toolbar =
+		gtk_ui_manager_get_widget (ev_window->priv->ui_manager,
+					   "/ToolBar");
+	gtk_box_pack_start (GTK_BOX (toolbar_dock), ev_window->priv->toolbar,
+			    TRUE, TRUE, 0);
+	gtk_widget_show (ev_window->priv->toolbar);
+
+	ev_window->priv->exit_fullscreen_toolbar =
+		gtk_ui_manager_get_widget (ev_window->priv->ui_manager,
+					   "/LeaveFullscreenToolbar");
+	gtk_box_pack_start (GTK_BOX (toolbar_dock),
+			    ev_window->priv->exit_fullscreen_toolbar,
+			    FALSE, FALSE, 0);
+	gtk_toolbar_set_show_arrow (GTK_TOOLBAR (ev_window->priv->exit_fullscreen_toolbar),
+				    FALSE);
+	gtk_toolbar_set_style (GTK_TOOLBAR (ev_window->priv->exit_fullscreen_toolbar),
+			       GTK_TOOLBAR_BOTH_HORIZ);
 
 	/* Add the main area */
 	ev_window->priv->hpaned = gtk_hpaned_new ();
@@ -1593,7 +1827,6 @@ ev_window_init (EvWindow *ev_window)
 			    TRUE, TRUE, 0);
 
 	ev_window->priv->sidebar = ev_sidebar_new ();
-	gtk_widget_show (ev_window->priv->sidebar);
 	gtk_paned_add1 (GTK_PANED (ev_window->priv->hpaned),
 			ev_window->priv->sidebar);
 
@@ -1624,15 +1857,19 @@ ev_window_init (EvWindow *ev_window)
 			ev_window->priv->scrolled_window);
 
 	ev_window->priv->view = ev_view_new ();
+	//ev_window->priv->page_view = ev_page_view_new ();
 	ev_window->priv->password_view = ev_password_view_new ();
 	g_signal_connect_swapped (ev_window->priv->password_view,
 				  "unlock",
 				  G_CALLBACK (ev_window_popup_password_dialog),
 				  ev_window);
 	gtk_widget_show (ev_window->priv->view);
+	//gtk_widget_show (ev_window->priv->page_view);
 	gtk_widget_show (ev_window->priv->password_view);
+
 	/* We own a ref on these widgets, as we can swap them in and out */
 	g_object_ref (ev_window->priv->view);
+	//g_object_ref (ev_window->priv->page_view);
 	g_object_ref (ev_window->priv->password_view);
 
 	gtk_container_add (GTK_CONTAINER (ev_window->priv->scrolled_window),
@@ -1651,7 +1888,6 @@ ev_window_init (EvWindow *ev_window)
 			  ev_window);
 
 	ev_window->priv->statusbar = gtk_statusbar_new ();
-	gtk_widget_show (ev_window->priv->statusbar);
 	gtk_box_pack_end (GTK_BOX (ev_window->priv->main_box),
 			  ev_window->priv->statusbar,
 			  FALSE, TRUE, 0);
@@ -1665,6 +1901,10 @@ ev_window_init (EvWindow *ev_window)
 			  ev_window->priv->find_bar,
 			  FALSE, TRUE, 0);
 
+	ev_window->priv->chrome = load_chrome ();
+	set_chrome_actions (ev_window);
+	update_chrome_visibility (ev_window);
+
 	/* Connect to find bar signals */
 	g_signal_connect (ev_window->priv->find_bar,
 			  "previous",
@@ -1694,9 +1934,6 @@ ev_window_init (EvWindow *ev_window)
 	g_signal_connect (ev_window, "window-state-event",
 			  G_CALLBACK (ev_window_state_event_cb),
 			  ev_window);
-	g_signal_connect (ev_window, "focus_out_event",
-			  G_CALLBACK (ev_window_focus_out_cb),
-			  ev_window);
 
 	/* Give focus to the scrolled window */
 	gtk_widget_grab_focus (ev_window->priv->scrolled_window);
