@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: main.c,v 1.6 2005/01/04 15:53:49 marco Exp $
+ *  $Id: main.c,v 1.7 2005/02/07 21:11:11 marco Exp $
  */
 
 #include "config.h"
@@ -29,6 +29,7 @@
 #include <libgnomevfs/gnome-vfs-utils.h>
 
 #include "ev-stock-icons.h"
+#include "ev-debug.h"
 
 static struct poptOption popt_options[] =
 {
@@ -83,6 +84,7 @@ main (int argc, char *argv[])
 
 	g_set_application_name (_("Evince Document Viewer"));
 
+	ev_debug_init ();
 	ev_stock_icons_init ();
 
 	g_object_get_property (G_OBJECT (program),
